"""Implementation of the Vista Gateway client using Mqtt."""

from typing import Optional
import logging

from http import HTTPStatus
from paho.mqtt.client import Client

from vista_gateway_client.vista_gateway_client import VistaGatewayClient
from vista_gateway_client.vista_gateway_exception import VistaGatewayException
from vista_gateway_client.gateway_package import GatewayPackage, ISO19848


class MqttClientImpl(VistaGatewayClient):
    """Implementation of the Vista Gateway client using Mqtt."""

    def __init__(
        self,
        api_key: Optional[str],
        endpoint: str,
        logger: Optional[logging.Logger] = None,
    ):
        self._api_key = api_key
        self._endpoint = endpoint
        self._client = Client(...).connect(endpoint)
        # self._client.base_url = endpoint
        self._logger = logger or logging.getLogger(__name__)

    def send(self, package: GatewayPackage) -> None:  # **kwargs
        """Send a package to the Vista Gateway API."""
        # cancellationToken: CancellationToken = default        
        response = None
        try:
            if isinstance(package, ISO19848.DataChannelList):
                content = package.to_json_dto()
                # external_id = package.external_id
                # correlation_id = package.correlation_id
                user_agent = package.user_agent
            elif isinstance(package, ISO19848.TimeSeriesData):
                content = package.to_json_dto()
                # external_id = package.external_id
                # correlation_id = package.correlation_id
                user_agent = package.user_agent
            else:
                raise ValueError("Unknown package type")
            
            headers = {
                "Content-Type": "application/json",
                "Authorization": f"ApiKey {self._api_key}",
                "User-Agent": user_agent or "Vista.Gateway.Client.HttpClient"
            }

            response = self._client.post(
                url=f"{self._client.base_url}/ISO19848",
                data=content,
                headers=headers,
            )

            if response.status_code != HTTPStatus.OK:
                reason = response.text
                raise VistaGatewayException(
                    "Failed to send package",
                    response.status_code,
                    reason,
                )
            
            return response

        except Exception as ex:
            self._logger.error("Failed to send package", exc_info=ex)
            if not isinstance(ex, VistaGatewayException):
                raise VistaGatewayException(
                    "Failed to send package",
                    response.status_code if response else None,
                    response.reason if response else None,
                    ex
                )
